@@ -1,5 +1,3 @@
-// TODO: split the parsing code from teh gen code
-
 use super::*;
 
 #[derive(Clone, Eq, Ord, PartialEq, PartialOrd, Default)]
@@ -12,14 +10,6 @@ pub struct Signature {
 }
 
 impl Signature {
-    pub fn definition(&self, include: TypeInclude) -> Vec<TypeEntry> {
-        self.kind.definition(include)
-    }
-
-    pub fn dependencies(&self, include: TypeInclude) -> Vec<TypeEntry> {
-        self.kind.dependencies(include)
-    }
-
     pub fn is_blittable(&self) -> bool {
         self.pointers > 0 || self.kind.is_blittable()
     }
@@ -43,4 +33,12 @@ impl Signature {
             _ => false,
         }
     }
+
+    pub fn size(&self) -> usize {
+        if self.pointers > 0 {
+            1
+        } else {
+            self.kind.size()
+        }
+    }
 }
