@@ -53,26 +53,6 @@ impl TypeDef {
             .expect("`Invoke` method not found")
     }
 
-    pub fn definition(&self, mut include: TypeInclude) -> Vec<TypeEntry> {
-        // The `Windows.Foundation` namespace includes supporting types that should
-        // always be fully-defined when included because their methods are almost
-        // always needed.
-        if include != TypeInclude::Full && self.namespace().starts_with("Windows.Foundation") {
-            include = TypeInclude::Full;
-        }
-
-        let mut definition = vec![TypeEntry {
-            include,
-            def: ElementType::TypeDef(self.clone()),
-        }];
-
-        for generic in &self.generics {
-            definition.append(&mut generic.definition(include));
-        }
-
-        definition
-    }
-
     pub fn default_interface(&self) -> Option<Self> {
         for interface in self.interface_impls() {
             if interface.is_default() {
@@ -104,112 +84,144 @@ impl TypeDef {
             return true;
         }
 
-        self.fields().any(|field| field.signature().is_packed())
+        self.fields()
+            .any(|field| field.signature(Some(self)).is_packed())
+    }
+
+    pub fn size(&self) -> usize {
+        if self.kind() == TypeKind::Struct {
+            self.fields()
+                .fold(0, |sum, field| sum + field.signature(Some(self)).size())
+        } else {
+            1
+        }
     }
 
     pub fn is_handle(&self) -> bool {
         self.has_attribute("NativeTypedefAttribute")
     }
 
-    pub fn dependencies(&self, include: TypeInclude) -> Vec<TypeEntry> {
+    pub fn include_dependencies(&self, include: TypeInclude) {
         match self.kind() {
             TypeKind::Interface => {
                 if include == TypeInclude::Minimal {
-                    return Vec::new();
+                    return;
                 }
 
-                let interfaces = self.interfaces().map(|i| i.definition(include));
-                let methods = self.methods().map(|m| m.dependencies());
-                interfaces.chain(methods).flatten().collect()
+                self.interfaces()
+                    .for_each(|i| i.include_definition(include));
+                self.methods().for_each(|m| m.include_dependencies());
             }
             TypeKind::Class => {
                 if include == TypeInclude::Minimal {
                     if let Some(default_interface) = self.default_interface() {
-                        return default_interface.definition(TypeInclude::Minimal);
-                    } else {
-                        return Vec::new();
+                        default_interface.include_definition(TypeInclude::Minimal);
                     }
+
+                    return;
                 }
 
-                let generics = self
-                    .generics
+                // TODO: test for this?
+                self.generics
                     .iter()
-                    .map(|g| g.definition(TypeInclude::Minimal));
-                let interfaces = self.interfaces().map(|i| i.definition(TypeInclude::Full));
-                let bases = self.bases().map(|b| b.definition(TypeInclude::Full));
+                    .for_each(|g| g.include_definition(TypeInclude::Minimal));
 
-                let factories = self.attributes().filter_map(|attribute| {
-                    match attribute.name() {
+                self.interfaces()
+                    .for_each(|i| i.include_definition(TypeInclude::Full));
+                self.bases()
+                    .for_each(|b| b.include_definition(TypeInclude::Full));
+
+                self.attributes()
+                    .for_each(|attribute| match attribute.name() {
                         "StaticAttribute" | "ActivatableAttribute" | "ComposableAttribute" => {
                             for (_, arg) in attribute.args() {
                                 if let ConstantValue::TypeDef(def) = arg {
-                                    return Some(TypeEntry {
-                                        include: TypeInclude::Full,
-                                        def: ElementType::TypeDef(def),
-                                    });
+                                    def.include_definition(TypeInclude::Full);
                                 }
                             }
                         }
                         _ => {}
-                    }
-
-                    None
-                });
-
-                generics
-                    .chain(interfaces)
-                    .chain(bases)
-                    .flatten()
-                    .chain(factories)
-                    .collect()
+                    });
             }
-            TypeKind::Enum => Vec::new(),
             TypeKind::Struct => {
-                let reader = TypeReader::get();
-                let mut dependencies = vec![];
+                self.fields()
+                    .for_each(|f| f.include_definition(Some(self), TypeInclude::Minimal));
 
-                // TODO: find better way to manage this
-                let type_name = self.type_name();
-
-                if type_name == TypeName::BSTR {
-                    dependencies.push(TypeEntry {
-                        include: TypeInclude::Minimal,
-                        def: reader.expect_type(TypeName::SysFreeString),
-                    });
-                    dependencies.push(TypeEntry {
-                        include: TypeInclude::Minimal,
-                        def: reader.expect_type(TypeName::SysAllocStringLen),
-                    });
-                    dependencies.push(TypeEntry {
-                        include: TypeInclude::Minimal,
-                        def: reader.expect_type(TypeName::SysStringLen),
-                    });
-                } else if type_name == TypeName::Matrix3x2 {
-                    dependencies.push(TypeEntry {
-                        include: TypeInclude::Minimal,
-                        def: reader.expect_type(TypeName::D2D1MakeRotateMatrix),
-                    });
-                } else {
-                    dependencies.extend(
-                        self.fields()
-                            .map(|f| f.definition(TypeInclude::Minimal))
-                            .flatten(),
-                    );
-
-                    if let Some(dependency) = self.is_convertible_to() {
-                        dependencies.push(TypeEntry {
-                            include: TypeInclude::Minimal,
-                            def: ElementType::TypeDef(dependency),
-                        });
-                    }
+                if let Some(dependency) = self.is_convertible_to() {
+                    dependency.include_definition(TypeInclude::Minimal);
                 }
-
-                dependencies
             }
-            TypeKind::Delegate => self.invoke_method().dependencies(),
+            TypeKind::Delegate => self.invoke_method().include_dependencies(),
+            TypeKind::Enum => {}
         }
     }
 
+    pub fn include_definition(&self, include: TypeInclude) {
+        let type_name = self.type_name();
+
+        if type_name.namespace().is_empty() {
+            self.include_dependencies(TypeInclude::Minimal);
+        } else {
+            TypeReader::get_mut().include_type_name(type_name, include);
+
+            for generic in &self.generics {
+                generic.include_definition(include);
+            }
+        }
+    }
+
+    pub fn struct_features(
+        &self,
+        features: &mut BTreeSet<&'static str>,
+        keys: &mut std::collections::HashSet<Row>,
+    ) {
+        if !keys.insert(self.row.clone()) {
+            return;
+        }
+
+        let namespace = self.namespace();
+
+        if !namespace.is_empty() {
+            features.insert(self.namespace());
+        }
+
+        match self.kind() {
+            TypeKind::Class => {
+                if let Some(def) = self.default_interface() {
+                    features.insert(def.namespace());
+                }
+            }
+            TypeKind::Struct => {
+                self.fields()
+                    .for_each(|def| def.struct_features(Some(self), features, keys));
+
+                if let Some(def) = self.is_convertible_to() {
+                    // TODO: wonky
+                    features.insert(def.type_name().namespace);
+                }
+            }
+            TypeKind::Delegate => self
+                .invoke_method()
+                .signature(&[])
+                .struct_features(features, keys),
+            _ => {}
+        }
+    }
+
+    pub fn method_features(
+        &self,
+        features: &mut BTreeSet<&'static str>,
+        keys: &mut std::collections::HashSet<Row>,
+    ) {
+        features.insert(self.namespace());
+
+        for generic in &self.generics {
+            generic.method_features(features, keys);
+        }
+
+        self.struct_features(features, keys);
+    }
+
     pub fn is_udt(&self) -> bool {
         // TODO: should this just check whether the struct has > 1 fields rather than is_handle?
         self.kind() == TypeKind::Struct && !self.is_handle()
@@ -239,7 +251,8 @@ impl TypeDef {
         if self.is_explicit() {
             true
         } else {
-            self.fields().any(|f| f.signature().has_explicit())
+            self.fields()
+                .any(|f| f.signature(Some(self)).has_explicit())
         }
     }
 
@@ -266,7 +279,7 @@ impl TypeDef {
 
                 for field in self.fields() {
                     result.push(';');
-                    result.push_str(&field.signature().kind.type_signature());
+                    result.push_str(&field.signature(Some(self)).kind.type_signature());
                 }
 
                 result.push(')');
@@ -287,14 +300,14 @@ impl TypeDef {
             if let Some(constant) = field.constant() {
                 return constant.value_type();
             } else {
-                return field.signature().kind;
+                return field.signature(Some(self)).kind;
             }
         }
 
         unimplemented!();
     }
 
-    pub fn interface_signature(&self) -> String {
+    fn interface_signature(&self) -> String {
         let guid = self.guid();
 
         if self.generics.is_empty() {
@@ -436,19 +449,11 @@ impl TypeDef {
         })
     }
 
-    pub fn is_convertible_to(&self) -> Option<TypeDef> {
+    pub fn is_convertible_to(&self) -> Option<&ElementType> {
         self.attributes().find_map(|attribute| {
             if attribute.name() == "AlsoUsableForAttribute" {
                 if let Some((_, ConstantValue::String(name))) = attribute.args().get(0) {
-                    return TypeReader::get()
-                        .get_type((self.namespace(), name.as_str()))
-                        .and_then(|row| {
-                            if let ElementType::TypeDef(def) = row {
-                                Some(def)
-                            } else {
-                                None
-                            }
-                        });
+                    return TypeReader::get().get_type((self.namespace(), name.as_str()));
                 }
             }
 
@@ -473,7 +478,7 @@ impl TypeDef {
                 if self.type_name() == TypeName::BSTR {
                     false
                 } else {
-                    self.fields().all(|f| f.is_blittable())
+                    self.fields().all(|f| f.is_blittable(Some(self)))
                 }
             }
             TypeKind::Enum => true,
@@ -511,8 +516,8 @@ impl TypeDef {
         (0, 0)
     }
 
-    pub fn guid(&self) -> Guid {
-        Guid::from_attributes(self.attributes()).expect("TypeDef::guid")
+    pub fn guid(&self) -> GUID {
+        GUID::from_attributes(self.attributes()).expect("TypeDef::guid")
     }
 
     pub fn is_nullable(&self) -> bool {
@@ -542,7 +547,7 @@ impl TypeDef {
     pub fn overridable_interfaces(&self) -> Vec<TypeDef> {
         self.interface_impls()
             .filter(|interface| interface.is_overridable())
-            .map(|interface| interface.interface().resolve())
+            .map(|interface| interface.interface().resolve(None))
             .chain(
                 self.bases()
                     .next()
@@ -558,13 +563,6 @@ impl TypeDef {
             .flat_map(|interface| interface.methods().map(|method| method.name()))
             .collect()
     }
-
-    // May need to "re-resolve" the TypeDef as it may point to an arch-specific
-    // definition. This lets the TypeTree be built for a specific architecture
-    // without accidentally pulling in the wrong definition.
-    pub fn resolve(&self) -> Self {
-        TypeReader::get().expect_type_def(self.type_name())
-    }
 }
 
 struct Bases(TypeDef);
