@@ -36,7 +36,7 @@ impl TypeName {
     pub const Delegate: Self = Self::from_const("System", "MulticastDelegate");
     pub const Struct: Self = Self::from_const("System", "ValueType");
     pub const Object: Self = Self::from_const("System", "Object");
-    pub const Guid: Self = Self::from_const("System", "Guid");
+    pub const GUID: Self = Self::from_const("System", "Guid");
     pub const Type: Self = Self::from_const("System", "Type");
     pub const Attribute: Self = Self::from_const("System", "Attribute");
     pub const IsConst: Self = Self::from_const("System.Runtime.CompilerServices", "IsConst");
@@ -61,8 +61,7 @@ impl TypeName {
     pub const IVectorView: Self = Self::from_const("Windows.Foundation.Collections", "IVectorView");
     pub const IVector: Self = Self::from_const("Windows.Foundation.Collections", "IVector");
 
-    pub const WIN32_ERROR: Self =
-        Self::from_const("Windows.Win32.System.Diagnostics.Debug", "WIN32_ERROR");
+    pub const WIN32_ERROR: Self = Self::from_const("Windows.Win32.Foundation", "WIN32_ERROR");
     pub const NTSTATUS: Self = Self::from_const("Windows.Win32.Foundation", "NTSTATUS");
     pub const BOOL: Self = Self::from_const("Windows.Win32.Foundation", "BOOL");
     pub const PWSTR: Self = Self::from_const("Windows.Win32.Foundation", "PWSTR");
@@ -79,12 +78,6 @@ impl TypeName {
         Self::from_const("Windows.Win32.System.SystemServices", "LARGE_INTEGER");
     pub const ULARGE_INTEGER: Self =
         Self::from_const("Windows.Win32.System.SystemServices", "ULARGE_INTEGER");
-    pub const SysFreeString: Self = Self::from_const("Windows.Win32.Foundation", "SysFreeString");
-    pub const SysAllocStringLen: Self =
-        Self::from_const("Windows.Win32.Foundation", "SysAllocStringLen");
-    pub const SysStringLen: Self = Self::from_const("Windows.Win32.Foundation", "SysStringLen");
-    pub const D2D1MakeRotateMatrix: Self =
-        Self::from_const("Windows.Win32.Graphics.Direct2D", "D2D1MakeRotateMatrix");
     pub const IRestrictedErrorInfo: Self =
         Self::from_const("Windows.Win32.System.WinRT", "IRestrictedErrorInfo");
 
