@@ -92,6 +92,8 @@ impl TypeReader {
             }
         }
 
+        // TODO: remove Windows.Win32.Interop namespace
+
         Self { nested, types }
     }
 
@@ -117,36 +119,46 @@ impl TypeReader {
         self.import_type_include(namespace, name, TypeInclude::Full)
     }
 
-    fn import_type_dependencies(&mut self, def: &ElementType, include: TypeInclude) {
-        for entry in def.dependencies(include) {
-            let type_name = entry.def.type_name();
-
-            // If def.namespace is empty it means its a nested type and we need to find its dependencies to avoid type slicing.
-            if type_name.namespace.is_empty() {
-                self.import_type_dependencies(&entry.def, TypeInclude::Minimal);
-            } else {
-                self.import_type_include(type_name.namespace, type_name.name, entry.include);
-            }
-        }
+    pub fn include_type_name<T: HasTypeName>(
+        &mut self,
+        type_name: T,
+        include: TypeInclude,
+    ) -> bool {
+        self.import_type_include(type_name.namespace(), type_name.name(), include)
     }
 
-    fn import_type_include(&mut self, namespace: &str, name: &str, include: TypeInclude) -> bool {
-        assert!(!namespace.is_empty());
+    fn import_type_include(
+        &mut self,
+        namespace: &str,
+        name: &str,
+        mut include: TypeInclude,
+    ) -> bool {
+        // The `Windows.Foundation` namespace includes supporting types that should
+        // always be fully-defined when included because their methods are almost
+        // always needed.
+        if include != TypeInclude::Full && namespace.starts_with("Windows.Foundation") {
+            include = TypeInclude::Full;
+        }
+
         if let Some(entry) = self
             .types
             .get_namespace_mut(namespace)
             .and_then(|tree| tree.get_type_mut(name))
         {
-            let copy = entry.def.clone();
-
             if include == TypeInclude::Full {
                 if entry.include != TypeInclude::Full {
                     entry.include = TypeInclude::Full;
-                    self.import_type_dependencies(&copy, include);
+                    entry
+                        .def
+                        .iter()
+                        .for_each(|def| def.include_dependencies(include));
                 }
             } else if entry.include == TypeInclude::None {
                 entry.include = TypeInclude::Minimal;
-                self.import_type_dependencies(&copy, include);
+                entry
+                    .def
+                    .iter()
+                    .for_each(|def| def.include_dependencies(include));
             }
 
             true
@@ -162,28 +174,18 @@ impl TypeReader {
         self.nested.get(&enclosing.row)
     }
 
-    pub fn get_type<T: HasTypeName>(&'static self, type_name: T) -> Option<ElementType> {
+    pub fn get_type<T: HasTypeName>(&'static self, type_name: T) -> Option<&ElementType> {
         self.types
             .get_namespace(type_name.namespace())
             .and_then(|tree| tree.get_type(type_name.name()))
-            .map(|entry| entry.def.clone())
-    }
-
-    pub fn expect_type<T: HasTypeName>(&'static self, type_name: T) -> ElementType {
-        self.get_type(type_name).unwrap_or_else(|| {
-            panic!(
-                "Expected type not found `{}.{}`",
-                type_name.namespace(),
-                type_name.name()
-            )
-        })
+            .and_then(|entry| entry.def.first())
     }
 
     pub fn expect_type_def<T: HasTypeName>(&'static self, type_name: T) -> TypeDef {
         self.get_type(type_name)
             .and_then(|def| {
                 if let ElementType::TypeDef(def) = def {
-                    Some(def)
+                    Some(def.clone())
                 } else {
                     None
                 }
@@ -197,26 +199,35 @@ impl TypeReader {
             })
     }
 
-    pub fn expect_type_ref(&'static self, type_ref: &TypeRef) -> TypeDef {
-        if let ResolutionScope::TypeRef(scope) = type_ref.scope() {
-            self.nested[&scope.resolve().row]
-                .get(type_ref.name())
-                .unwrap_or_else(|| {
-                    panic!(
-                        "Could not find nested type `{}` in `{}`",
-                        type_ref.name(),
-                        scope.type_name()
-                    )
-                })
-                .clone()
-        } else {
-            self.expect_type_def(type_ref.type_name())
+    pub fn expect_type_ref(
+        &'static self,
+        enclosing: Option<&TypeDef>,
+        type_ref: &TypeRef,
+    ) -> TypeDef {
+        let type_name = type_ref.type_name();
+
+        if let Some(enclosing) = enclosing {
+            if type_name.namespace.is_empty() {
+                return self.nested[&enclosing.row]
+                    .get(type_name.name)
+                    .unwrap_or_else(|| {
+                        panic!(
+                            "Could not find nested type `{}` in `{}`",
+                            type_name.name,
+                            enclosing.type_name(),
+                        )
+                    })
+                    .clone();
+            }
         }
+
+        self.expect_type_def(type_name)
     }
 
     pub fn signature_from_blob(
         &'static self,
         blob: &mut Blob,
+        enclosing: Option<&TypeDef>,
         generics: &[ElementType],
     ) -> Option<Signature> {
         let is_const = blob
@@ -238,7 +249,7 @@ impl TypeReader {
             pointers += 1;
         }
 
-        let kind = self.type_from_blob(blob, generics);
+        let kind = self.type_from_blob(blob, enclosing, generics);
 
         Some(Signature {
             kind,
@@ -252,11 +263,12 @@ impl TypeReader {
     pub fn type_from_code(
         &'static self,
         code: &TypeDefOrRef,
+        enclosing: Option<&TypeDef>,
         generics: &[ElementType],
     ) -> ElementType {
         if let TypeDefOrRef::TypeSpec(def) = code {
             let mut blob = def.blob();
-            return self.type_from_blob(&mut blob, generics);
+            return self.type_from_blob(&mut blob, enclosing, generics);
         }
 
         let full_name = code.type_name();
@@ -273,10 +285,15 @@ impl TypeReader {
             }
         }
 
-        code.resolve().into()
+        code.resolve(enclosing).into()
     }
 
-    pub fn type_from_blob(&'static self, blob: &mut Blob, generics: &[ElementType]) -> ElementType {
+    pub fn type_from_blob(
+        &'static self,
+        blob: &mut Blob,
+        enclosing: Option<&TypeDef>,
+        generics: &[ElementType],
+    ) -> ElementType {
         let code = blob.read_unsigned();
 
         if let Some(code) = ElementType::from_code(code) {
@@ -286,6 +303,7 @@ impl TypeReader {
         match code {
             0x11 | 0x12 => self.type_from_code(
                 &TypeDefOrRef::decode(blob.file, blob.read_unsigned()),
+                enclosing,
                 generics,
             ),
             0x13 => generics
@@ -293,7 +311,7 @@ impl TypeReader {
                 .unwrap_or(&ElementType::Void)
                 .clone(),
             0x14 => {
-                let kind = self.signature_from_blob(blob, generics).unwrap();
+                let kind = self.signature_from_blob(blob, enclosing, generics).unwrap();
                 let _rank = blob.read_unsigned();
                 let _bounds_count = blob.read_unsigned();
                 let bounds = blob.read_unsigned();
@@ -302,11 +320,13 @@ impl TypeReader {
             0x15 => {
                 blob.read_unsigned();
 
-                let mut def = TypeDefOrRef::decode(blob.file, blob.read_unsigned()).resolve();
+                let mut def =
+                    TypeDefOrRef::decode(blob.file, blob.read_unsigned()).resolve(enclosing);
                 let args = blob.read_unsigned();
 
                 for _ in 0..args {
-                    def.generics.push(self.type_from_blob(blob, generics));
+                    def.generics
+                        .push(self.type_from_blob(blob, enclosing, generics));
                 }
 
                 ElementType::TypeDef(def)
@@ -329,7 +349,7 @@ fn is_well_known(type_name: TypeName) -> bool {
 const REMAP_TYPES: [(TypeName, TypeName); 1] = [(TypeName::D2D_MATRIX_3X2_F, TypeName::Matrix3x2)];
 
 const WELL_KNOWN_TYPES: [(TypeName, ElementType); 9] = [
-    (TypeName::Guid, ElementType::Guid),
+    (TypeName::GUID, ElementType::GUID),
     (TypeName::IUnknown, ElementType::IUnknown),
     (TypeName::HResult, ElementType::HRESULT),
     (TypeName::HRESULT, ElementType::HRESULT),
